@@ -1,7 +1,4 @@
-/* mimic_json.c
- * Copyright (c) 2012, 2017, Peter Ohler
- * All rights reserved.
- */
+// Copyright (c) 2012, 2017 Peter Ohler. All rights reserved.
 
 #include "oj.h"
 #include "encode.h"
@@ -367,7 +364,6 @@ mimic_generate_core(int argc, VALUE *argv, Options copts) {
     struct _out	out;
     VALUE	rstr;
 
-    // TBD
     memset(buf, 0, sizeof(buf));
 
     out.buf = buf;
@@ -513,7 +509,6 @@ mimic_parse_core(int argc, VALUE *argv, VALUE self, bool bang) {
     pi.options.create_ok = No;
     pi.options.allow_nan = (bang ? Yes : No);
     pi.options.nilnil = No;
-    //pi.options.bigdec_load = FloatDec;
     pi.options.bigdec_load = RubyDec;
     pi.options.mode = CompatMode;
     pi.max_depth = 100;
@@ -564,6 +559,9 @@ mimic_parse_core(int argc, VALUE *argv, VALUE self, bool bang) {
 		pi.options.array_class = v;
 	    }
 	}
+	if (Qtrue == rb_funcall(ropts, oj_has_key_id, 1, oj_decimal_class_sym)) {
+	    pi.options.compat_bigdec = (oj_bigdecimal_class == rb_hash_lookup(ropts, oj_decimal_class_sym));
+	}
 	v = rb_hash_lookup(ropts, oj_max_nesting_sym);
 	if (Qtrue == v) {
 	    pi.max_depth = 100;
@@ -688,6 +686,7 @@ static struct _options	mimic_object_to_json_options = {
     RubyTime,	// time_format
     No,		// bigdec_as_num
     RubyDec,	// bigdec_load
+    false,	// compat_bigdec
     No,		// to_hash
     No,		// to_json
     No,		// as_json
