@@ -1,7 +1,4 @@
-/* parse.c
- * Copyright (c) 2013, Peter Ohler
- * All rights reserved.
- */
+// Copyright (c) 2013 Peter Ohler. All rights reserved.
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -215,18 +212,6 @@ read_escaped_str(ParseInfo pi, const char *start) {
 	    case '"':	buf_append(&buf, '"');	break;
 	    case '/':	buf_append(&buf, '/');	break;
 	    case '\\':	buf_append(&buf, '\\');	break;
-	    case '\'':
-		// The json gem claims this is not an error despite the
-		// ECMA-404 indicating it is not valid.
-		if (CompatMode == pi->options.mode) {
-		    buf_append(&buf, '\'');
-		} else {
-		    pi->cur = s;
-		    oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "invalid escaped character");
-		    buf_cleanup(&buf);
-		    return;
-		}
-		break;
 	    case 'u':
 		s++;
 		if (0 == (code = read_hex(pi, s)) && err_has(&pi->err)) {
@@ -266,6 +251,12 @@ read_escaped_str(ParseInfo pi, const char *start) {
 		}
 		break;
 	    default:
+		// The json gem claims this is not an error despite the
+		// ECMA-404 indicating it is not valid.
+		if (CompatMode == pi->options.mode) {
+		    buf_append(&buf, *s);
+		    break;
+		}
 		pi->cur = s;
 		oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "invalid escaped character");
 		buf_cleanup(&buf);
@@ -394,8 +385,13 @@ read_num(ParseInfo pi) {
     ni.nan = 0;
     ni.neg = 0;
     ni.has_exp = 0;
-    ni.no_big = (FloatDec == pi->options.bigdec_load || FastDec == pi->options.bigdec_load || RubyDec == pi->options.bigdec_load);
-    ni.bigdec_load = pi->options.bigdec_load;
+    if (CompatMode == pi->options.mode) {
+	ni.no_big = !pi->options.compat_bigdec;
+	ni.bigdec_load = pi->options.compat_bigdec;
+    } else {
+	ni.no_big = (FloatDec == pi->options.bigdec_load || FastDec == pi->options.bigdec_load || RubyDec == pi->options.bigdec_load);
+	ni.bigdec_load = pi->options.bigdec_load;
+    }
 
     if ('-' == *pi->cur) {
 	pi->cur++;
@@ -520,7 +516,11 @@ read_num(ParseInfo pi) {
 	    ni.nan = 1;
 	}
     }
-    if (BigDec == pi->options.bigdec_load) {
+    if (CompatMode == pi->options.mode) {
+	if (pi->options.compat_bigdec) {
+	    ni.big = 1;
+	}
+    } else if (BigDec == pi->options.bigdec_load) {
 	ni.big = 1;
     }
     if (0 == parent) {
