@@ -1,7 +1,4 @@
-/* sparse.c
- * Copyright (c) 2013, Peter Ohler
- * All rights reserved.
- */
+// Copyright (c) 2013 Peter Ohler. All rights reserved.
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -228,17 +225,6 @@ read_escaped_str(ParseInfo pi) {
 	    case '"':	buf_append(&buf, '"');	break;
 	    case '/':	buf_append(&buf, '/');	break;
 	    case '\\':	buf_append(&buf, '\\');	break;
-	    case '\'':
-		// The json gem claims this is not an error despite the
-		// ECMA-404 indicating it is not valid.
-		if (CompatMode == pi->options.mode) {
-		    buf_append(&buf, '\'');
-		} else {
-		    oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "invalid escaped character");
-		    buf_cleanup(&buf);
-		    return;
-		}
-		break;
 	    case 'u':
 		if (0 == (code = read_hex(pi)) && err_has(&pi->err)) {
 		    buf_cleanup(&buf);
@@ -276,6 +262,12 @@ read_escaped_str(ParseInfo pi) {
 		}
 		break;
 	    default:
+		// The json gem claims this is not an error despite the
+		// ECMA-404 indicating it is not valid.
+		if (CompatMode == pi->options.mode) {
+		    buf_append(&buf, c);
+		    break;
+		}
 		oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "invalid escaped character");
 		buf_cleanup(&buf);
 		return;
@@ -408,8 +400,13 @@ read_num(ParseInfo pi) {
     ni.nan = 0;
     ni.neg = 0;
     ni.has_exp = 0;
-    ni.no_big = (FloatDec == pi->options.bigdec_load || FastDec == pi->options.bigdec_load || RubyDec == pi->options.bigdec_load);
-    ni.bigdec_load = pi->options.bigdec_load;
+    if (CompatMode == pi->options.mode) {
+	ni.no_big = !pi->options.compat_bigdec;
+	ni.bigdec_load = pi->options.compat_bigdec;
+    } else {
+	ni.no_big = (FloatDec == pi->options.bigdec_load || FastDec == pi->options.bigdec_load || RubyDec == pi->options.bigdec_load);
+	ni.bigdec_load = pi->options.bigdec_load;
+    }
 
     c = reader_get(&pi->rd);
     if ('-' == c) {
@@ -526,7 +523,11 @@ read_num(ParseInfo pi) {
 	    ni.nan = 1;
 	}
     }
-    if (BigDec == pi->options.bigdec_load) {
+    if (CompatMode == pi->options.mode) {
+	if (pi->options.compat_bigdec) {
+	    ni.big = 1;
+	}
+    } else if (BigDec == pi->options.bigdec_load) {
 	ni.big = 1;
     }
     add_num_value(pi, &ni);
@@ -549,15 +550,24 @@ read_nan(ParseInfo pi) {
     ni.infinity = 0;
     ni.nan = 1;
     ni.neg = 0;
-    ni.no_big = (FloatDec == pi->options.bigdec_load || FastDec == pi->options.bigdec_load || RubyDec == pi->options.bigdec_load);
-    ni.bigdec_load = pi->options.bigdec_load;
+    if (CompatMode == pi->options.mode) {
+	ni.no_big = !pi->options.compat_bigdec;
+	ni.bigdec_load = pi->options.compat_bigdec;
+    } else {
+	ni.no_big = (FloatDec == pi->options.bigdec_load || FastDec == pi->options.bigdec_load || RubyDec == pi->options.bigdec_load);
+	ni.bigdec_load = pi->options.bigdec_load;
+    }
 
     if ('a' != reader_get(&pi->rd) ||
 	('N' != (c = reader_get(&pi->rd)) && 'n' != c)) {
 	oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "not a number or other value");
 	return;
     }
-    if (BigDec == pi->options.bigdec_load) {
+    if (CompatMode == pi->options.mode) {
+	if (pi->options.compat_bigdec) {
+	    ni.big = 1;
+	}
+    } else if (BigDec == pi->options.bigdec_load) {
 	ni.big = 1;
     }
     add_num_value(pi, &ni);
@@ -747,8 +757,15 @@ oj_sparse2(ParseInfo pi) {
 		ni.infinity = 0;
 		ni.nan = 1;
 		ni.neg = 0;
-		ni.no_big = (FloatDec == pi->options.bigdec_load || RubyDec == pi->options.bigdec_load || FastDec == pi->options.bigdec_load);
-		ni.bigdec_load = pi->options.bigdec_load;
+		if (CompatMode == pi->options.mode) {
+		    ni.no_big = !pi->options.compat_bigdec;
+		    ni.bigdec_load = pi->options.compat_bigdec;
+		} else {
+		    ni.no_big = (FloatDec == pi->options.bigdec_load ||
+				 FastDec == pi->options.bigdec_load ||
+				 RubyDec == pi->options.bigdec_load);
+		    ni.bigdec_load = pi->options.bigdec_load;
+		}
 		add_num_value(pi, &ni);
 	    } else {
 		oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "invalid token");
