@@ -236,6 +236,12 @@ class CompatJuice < Minitest::Test
     assert_equal({"a\nb" => true, "c\td" => false}, obj)
   end
 
+  def test_invalid_escapes_handled
+    json = '{"subtext":"\"404er\” \w \k \3 \a"}'
+    obj = Oj.compat_load(json)
+    assert_equal({"subtext" => "\"404er” w k 3 a"}, obj)
+  end
+
   def test_hash_escaping
     json = Oj.to_json({'<>' => '<>'}, mode: :compat)
     assert_equal(json, '{"<>":"<>"}')
@@ -271,12 +277,19 @@ class CompatJuice < Minitest::Test
   # BigDecimal
   def test_bigdecimal
     # BigDecimals are dumped as strings and can not be restored to the
-    # original value.
+    # original value without using an undocumented feature of the JSON gem.
     json = Oj.dump(BigDecimal('3.14159265358979323846'))
     # 2.4.0 changes the exponent to lowercase
     assert_equal('"0.314159265358979323846e1"', json.downcase)
   end
 
+  def test_decimal_class
+    big = BigDecimal('3.14159265358979323846')
+    # :decimal_class is the undocumented feature.
+    json = Oj.load('3.14159265358979323846', mode: :compat, decimal_class: BigDecimal)
+    assert_equal(big, json)
+  end
+
   def test_infinity
     assert_raises(Oj::ParseError) { Oj.load('Infinity', :mode => :strict) }
     x = Oj.load('Infinity', :mode => :compat)
